package main

import (
	"context"
	"fmt"
	"os"

	dataplanev1 "github.com/openstack-k8s-operators/dataplane-operator/api/v1beta1"
	"github.com/openstack-k8s-operators/lib-common/modules/common/secret"
	coreV1 "k8s.io/api/core/v1"
	metaV1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/runtime/schema"
	"k8s.io/client-go/kubernetes"
	"k8s.io/client-go/kubernetes/scheme"
	coreV1Types "k8s.io/client-go/kubernetes/typed/core/v1"
	"k8s.io/client-go/rest"
	"k8s.io/client-go/tools/clientcmd"
)

var secretsClient coreV1Types.SecretInterface
var restClient rest.Interface

func InitClients(namespace string) {
	kubeconfig := os.Getenv("HOME") + "/.kube/config"
	config, err := clientcmd.BuildConfigFromFlags("", kubeconfig)
	if err != nil {
		panic(err.Error())
	}
	clientset, err := kubernetes.NewForConfig(config)
	if err != nil {
		panic(err.Error())
	}
	secretsClient = clientset.CoreV1().Secrets(namespace)

	config.ContentConfig.GroupVersion = &schema.GroupVersion{Group: "dataplane.openstack.org", Version: "v1beta1"}
	config.APIPath = "/apis"
	config.NegotiatedSerializer = scheme.Codecs.WithoutConversion()
	config.UserAgent = rest.DefaultKubernetesUserAgent()

	restClient, err = rest.RESTClientFor(config)
	if err != nil {
		panic(err.Error())
	}
}

func GetCertSecrets(nodesetName string, namespace string) *coreV1.SecretList {
	opts := metaV1.ListOptions{
		LabelSelector: fmt.Sprintf("osdpns=%s", nodesetName),
	}
	certSecrets, err := secretsClient.List(context.TODO(), opts)
	if err != nil {
		panic(err.Error())
	}
	return certSecrets
}

func GetNodeSet(nodesetName string, namespace string) dataplanev1.OpenStackDataPlaneNodeSet {
	result := dataplanev1.OpenStackDataPlaneNodeSet{}
	err := restClient.
		Get().
		Namespace(namespace).
		Resource("openstackdataplanenodesets").
		Name(nodesetName).
		Do(context.TODO()).
		Into(&result)
	if err != nil {
		panic(err.Error())
	}
	return result
}

func ValidateCertSecrets(certSecrets *coreV1.SecretList, nodeset dataplanev1.OpenStackDataPlaneNodeSet) bool {
	valid := true
	for _, sec := range certSecrets.Items {
		secretHash, err := secret.Hash(&sec)
		if err != nil {
			fmt.Printf("Unable to get hash for %s\n", sec.Name)
			valid = false
			break
		}
		nodesetHash, ok := nodeset.Status.SecretHashes[sec.Name]
		if ok {
			if secretHash != nodesetHash {
				fmt.Printf("Hashes do not match for %s\n", sec.Name)
				valid = false
			}
		} else {
			// There may be old secrets which aren't cleaned up -
			// Just note, but do not fail here.
			fmt.Printf("Hash not found in nodeset for %s\n", sec.Name)
		}
	}
	return valid
}

func main() {
	namespace := os.Args[1]
	nodesetName := os.Args[2]
	InitClients(namespace)
	certSecrets := GetCertSecrets(nodesetName, namespace)
	nodeset := GetNodeSet(nodesetName, namespace)
	valid := ValidateCertSecrets(certSecrets, nodeset)
	if valid {
		os.Exit(0)
	} else {
		os.Exit(1)
	}
}
